
#include "Talky.h"
//---------------------------------------------------------------------------
DEFCALL CTALKY::CTALKY(BOOL Init)
{
 if(Init)Initialize();
   else this->SetDefaults();
}
//---------------------------------------------------------------------------
DEFCALL CTALKY::~CTALKY()
{
 UnInitialize();
}
//---------------------------------------------------------------------------
void DEFCALL CTALKY::SetDefaults(void)
{
 Voice        = NULL;
 VoicesList   = NULL;
 FileStream   = NULL;
 MemoryStream = NULL;
 AudioFormat  = NULL;
 VConfigSize  = NULL;
 PrevVoiceBuf = NULL;
 SVoicesList  = NULL;
 SpeakTo      = SPKTOSNDC;
 Asynchr      = false;
}
//---------------------------------------------------------------------------
void DEFCALL CTALKY::UnInitialize(void)
{
 // Use 'IsBadReadPtr' for checks (for some reason the Pointers may become invalid!)
 if(!IsBadReadPtr(Voice,4))Voice->Release();
 if(!IsBadReadPtr(VoicesList,4))VoicesList->Release();
 if(!IsBadReadPtr(FileStream,4))FileStream->Release();
 if(!IsBadReadPtr(MemoryStream,4))MemoryStream->Release();
 if(!IsBadReadPtr(AudioFormat,4))AudioFormat->Release();
 this->SetDefaults();
 //CoUninitialize();          // Don`t do this !!!
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::Initialize(DWORD OutTgt, BOOL Async)
{
 DWORD Result;

 this->UnInitialize();
 if((OutTgt!=SPKTOSNDC)&&(OutTgt!=SPKTOMEMB)&&(OutTgt!=SPKTOWAVF))return -1;
 Asynchr       = Async;
 SpeakTo       = OutTgt;
 DefHeapHandle = GetProcessHeap();
 FastZeroMemory(&VoiceString, sizeof(VoiceString));
 FastZeroMemory(&VoiceConfig, sizeof(VoiceConfig)); 
 FastZeroMemory(&WavFilePath, sizeof(WavFilePath));
 Result = CoInitializeEx(NULL,COINIT_MULTITHREADED|COINIT_SPEED_OVER_MEMORY);
 Result = CoCreateInstance(CLSID_SpAudioFormat, NULL, CLSCTX_ALL, __uuidof(ISpeechAudioFormat), (void **)&AudioFormat); // CoCreateInstance(CLSID_SpAudioFormat, NULL, CLSCTX_ALL, __uuidof(*AudioFormat), (void **)&AudioFormat);
 switch(SpeakTo)
  {
   case SPKTOSNDC:
     break;
   case SPKTOMEMB:
     Result = CoCreateInstance(CLSID_SpMemoryStream, NULL, CLSCTX_ALL, __uuidof(ISpeechMemoryStream), (void **)&MemoryStream); // CoCreateInstance(CLSID_SpMemoryStream, NULL, CLSCTX_ALL, __uuidof(*MemoryStream), (void **)&MemoryStream);
     break;
   case SPKTOWAVF:
     Result = CoCreateInstance(CLSID_SpFileStream, NULL, CLSCTX_ALL, __uuidof(ISpeechFileStream), (void **)&FileStream);  // CoCreateInstance(CLSID_SpFileStream, NULL, CLSCTX_ALL, __uuidof(*FileStream), (void **)&FileStream);
     break;
   default: return -1;
  }
 return Result;
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::SpeakText(LPSTR Text)
{                                  
 PWSTR   WText       = NULL;
 DWORD   WBufferLen  = NULL;
 DWORD   SpeechFlags = SPF_PURGEBEFORESPEAK;
 VARIANT VValue00;
 
 FastZeroMemory(&VValue00,sizeof(VARIANT));
 VValue00.vt = VT_I4;
 WBufferLen  = (lstrlen(Text)*2)+(VConfigSize*2)+16;
 WText       = (PWSTR)HeapAlloc(DefHeapHandle, 0, WBufferLen);
 lstrcpyW(WText, (PWSTR)&VoiceConfig);
 MultiByteToWideChar(CP_ACP, 0, Text, -1, (PWSTR)&WText[VConfigSize], WBufferLen);
 if(Asynchr)SpeechFlags = (SpeechFlags | SPF_ASYNC);
 switch(SpeakTo)
  {
   case SPKTOSNDC:
     Voice->put_AllowAudioOutputFormatChangesOnNextSet(TRUE);
     Voice->putref_AudioOutput(NULL);
     Voice->Speak(WText, (SpeechVoiceSpeakFlags)SpeechFlags, NULL);   
     break;
   case SPKTOMEMB:
     MemoryStream->Seek(VValue00,SSSPTRelativeToStart,&VValue00);
     Voice->put_AllowAudioOutputFormatChangesOnNextSet(TRUE);
     Voice->putref_AudioOutputStream(MemoryStream);  
     Voice->Speak(WText, (SpeechVoiceSpeakFlags)SpeechFlags, NULL);
     break;
   case SPKTOWAVF:
     DeleteFileW((PWSTR)&WavFilePath);
     FileStream->Open((PWSTR)&WavFilePath, SSFMCreateForWrite, FALSE);  // Default: 22KHz, 16Bit, Mono
     Voice->put_AllowAudioOutputFormatChangesOnNextSet(TRUE);
     Voice->putref_AudioOutputStream(FileStream);  
     Voice->Speak(WText, (SpeechVoiceSpeakFlags)SpeechFlags, NULL);
     FileStream->Close();
     break; 
  }
 if(WText)HeapFree(DefHeapHandle, 0, WText);   
 return 0;
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::SetVoice(LPSTR VoiceName)
{
 long               Count  = NULL;
 PWSTR              Descr  = NULL;
 ISpeechObjectToken *Token = NULL;
         
 if(Voice){Voice->Release();Voice = NULL;}  
 if(VoicesList){VoicesList->Release();VoicesList = NULL;}
 if(CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpeechVoice, (void **)&Voice))return 1;
 if(!Voice->GetVoices(L"", L"", &VoicesList))VoicesList->get_Count(&Count);
 MultiByteToWideChar(CP_ACP, 0, VoiceName, -1, (PWSTR)&VoiceString, sizeof(VoiceString));
 while(Count)
  {
   Count--;
   if(VoicesList->Item(Count, &Token))continue;
   Token->GetDescription(0, &Descr);  // No need release this Token ?
   if(!lstrcmpiW(Descr,(PWSTR)&VoiceString))
    {
     Count = Voice->putref_Voice(Token);
     if(VoicesList){VoicesList->Release();VoicesList = NULL;}
     return 0;    // Return HRESULT
    }  
  }
 FastZeroMemory(&VoiceString, sizeof(VoiceString));
 if(VoicesList){VoicesList->Release();VoicesList = NULL;}
 return 2;
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::SetVoiceConfig(LPSTR Config)
{
 VConfigSize = lstrlen(Config);   // Size in SYMBOLS!
 return (BOOL)(!MultiByteToWideChar(CP_ACP, 0, Config, -1, (PWSTR)&VoiceConfig, sizeof(VoiceConfig)));
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::SetWavFileName(LPSTR Filename)
{
 return (BOOL)(!MultiByteToWideChar(CP_ACP, 0, Filename, -1, (PWSTR)&WavFilePath, sizeof(WavFilePath)));
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::SetOutputFormat(DWORD OutputFmt)
{
 if(AudioFormat)AudioFormat->put_Type((SpeechAudioFormatType)OutputFmt);
   else return 1;
 if(FileStream)FileStream->putref_Format(AudioFormat); 
 if(MemoryStream)MemoryStream->putref_Format(AudioFormat);   
 return 0;
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::GetOutputFormat(DWORD *SamplePerSec, DWORD *BitsPerSample, DWORD *Channels)
{
 int  Fmt;
 long OutputFmt;
 int  WFormats[] = {8000,11025,12000,16000,22050,24000,32000,44100,48000};
 
 if(AudioFormat)AudioFormat->get_Type((SpeechAudioFormatType*)&OutputFmt);
   else return 1;
 if((OutputFmt < 4)||(OutputFmt > 39))return 2;     // SAFT8kHz8BitMono - SAFT48kHz16BitStereo
 OutputFmt -= 4;
 Fmt        = (4 - (OutputFmt % 4));
  
 if(SamplePerSec)(*SamplePerSec)   = WFormats[OutputFmt / 4]; 
 if(BitsPerSample)(*BitsPerSample) = (Fmt < 2)?(8):(16);
 if(Channels)(*Channels)           = ((Fmt == 0)||(Fmt == 2))?(1):(2);
 return 0;
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::GetSpeechBuffer(PVOID *Buffer, DWORD *Size)
{
 DWORD   Length;
 VARIANT VValue;
 
 if(!MemoryStream)return 1;
 FastZeroMemory(&VValue,sizeof(VARIANT));
 VValue.vt = VT_I4;
 MemoryStream->Seek(VValue,SSSPTRelativeToCurrentPosition,&VValue);
 Length    = VValue.ulVal;
 FastZeroMemory(&VValue,sizeof(VARIANT));
 VValue.vt = VT_ARRAY;
 MemoryStream->GetData(&VValue);
 if(!VValue.parray)return 2;
 if(Size)(*Size)     = Length;                              // VValue.parray->rgsabound->cElements;
 if(Buffer)(*Buffer) = VValue.parray->pvData;
 if(PrevVoiceBuf)VirtualFree(PrevVoiceBuf,0,MEM_RELEASE);   // {if(!VirtualFree(PrevVoiceBuf,0,MEM_RELEASE))DebugBreak();}
 PrevVoiceBuf = VValue.parray;
 return 0;
}
//---------------------------------------------------------------------------
DWORD DEFCALL CTALKY::GetVoicesList(LPSTR DestBuffer, DWORD BufferSize, PDWORD VCount)
{
 long                 Count    = NULL;
 PWSTR                Descr    = NULL;
 ISpeechVoice        *TmpVoice = NULL;
 ISpeechObjectToken  *Token    = NULL;
 ISpeechObjectTokens *TmpVoiceList;

 if(SVoicesList)HeapFree(DefHeapHandle,0,SVoicesList);
 SVoicesList = (wchar_t*)HeapAlloc(DefHeapHandle,HEAP_ZERO_MEMORY,16);
 if(CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpeechVoice, (void **)&TmpVoice))return 1;
 if(!TmpVoice->GetVoices(L"", L"", &TmpVoiceList))TmpVoiceList->get_Count(&Count);
 if(VCount)(*VCount) = Count;
 while(Count)
  {
   Count--;
   if(TmpVoiceList->Item(Count, &Token))continue;
   Token->GetDescription(0, &Descr);  // No need release this Token ?
   SVoicesList = (wchar_t*)HeapReAlloc(DefHeapHandle,HEAP_ZERO_MEMORY,SVoicesList,((lstrlenW(SVoicesList)+lstrlenW(Descr))*2)+32);
   lstrcatW(SVoicesList,Descr);
   lstrcatW(SVoicesList,L"\r\n");
  }
 if(TmpVoice){TmpVoice->Release();Voice = NULL;}
 if(TmpVoiceList){TmpVoiceList->Release();VoicesList = NULL;}
 WideCharToMultiByte(CP_ACP,0,SVoicesList,-1,DestBuffer,BufferSize,NULL,NULL);
 return 0;
}
//---------------------------------------------------------------------------
